//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "emsocket.h"
#include "opcodes.h"

CEMSocket::CEMSocket(void){
	limitenabled = false;
	downloadlimit = 0;
	connected = false;
	dataneeded = 0;
	datawaiting = 0;
	receivedpacket = 0;
	connected = false;
	readbuffer = 0;
	sendbuffer = 0;
	sendblen = 0;
	sent = 0;
}

CEMSocket::~CEMSocket(){
	ClearQueues();
	AsyncSelect(0);
}

void CEMSocket::ClearQueues(){
	for (POSITION pos = controlpacket_queue.GetHeadPosition();pos != 0;controlpacket_queue.GetNext(pos))
		delete controlpacket_queue.GetAt(pos);
	controlpacket_queue.RemoveAll();
	for (POSITION pos = standartpacket_queue.GetHeadPosition();pos != 0;standartpacket_queue.GetNext(pos))
		delete standartpacket_queue.GetAt(pos);
	standartpacket_queue.RemoveAll();
	if (receivedpacket){
		delete receivedpacket;
		receivedpacket = 0;
	}
	if (readbuffer){
		delete[] readbuffer;
		readbuffer = 0;
	}
	datawaiting = 0;
	dataneeded = 0;
	if (sendbuffer)
		delete[] sendbuffer;
	sendbuffer = 0;
	sendblen = 0;
	sent = 0;
}

void CEMSocket::OnClose(int nErrorCode){
	CAsyncSocket::OnClose(nErrorCode);
	ClearQueues();
};

BOOL CEMSocket::AsyncSelect(long lEvent){
	if (m_hSocket != INVALID_SOCKET)
		return CAsyncSocket::AsyncSelect(lEvent);
	return true;
}

void CEMSocket::OnReceive(int nErrorCode){
	if (nErrorCode){
		OnError(nErrorCode);
		return;
	}
	if (dataneeded == 0){
		dataneeded = 6;
		datawaiting = 0;
	}

	if (!receivedpacket){
		ASSERT (dataneeded < 7);
		uint32 len = Receive(header+datawaiting,dataneeded-datawaiting);
		if (len == (uint32)SOCKET_ERROR)
			return;
		datawaiting += len;
		if (datawaiting != dataneeded)
			return;
		receivedpacket = new Packet(header);
		dataneeded = 0;
		datawaiting = 0;

		switch (receivedpacket->prot){
			case OP_EDONKEYPROT:
			case OP_EXTENDEDPROT:
			case OP_EMULEPROT:
				break;
			default:
				delete receivedpacket;
				receivedpacket = 0;
				OnError(ERR_WRONGHEADER);
				return;
		}
		
		ASSERT(receivedpacket->size < 200000);
		if (receivedpacket->size > 200000){
			delete receivedpacket;
			receivedpacket = 0;
			OnError(ERR_TOOBIG);
			return;
		}
		dataneeded = receivedpacket->size;
		ASSERT (readbuffer == NULL);
		readbuffer = new char[receivedpacket->size+1];
	}
	ASSERT (readbuffer != NULL);
	ASSERT (receivedpacket != NULL);

	uint32 toreceive = dataneeded-datawaiting;
	if (limitenabled && (toreceive > downloadlimit))
		toreceive = downloadlimit;
	uint32 len = 0;
	if (toreceive)
		len = Receive(readbuffer+datawaiting,toreceive);
	if (len == (uint32)SOCKET_ERROR)
		return;
	if (limitenabled)
		downloadlimit -= len;
	datawaiting += len;

	if (datawaiting != dataneeded)
		return;
	dataneeded = 6;
	datawaiting = 0;	
	receivedpacket->pBuffer = readbuffer;
	readbuffer = 0;

	PacketReceived(receivedpacket);
	delete receivedpacket;
	receivedpacket = 0;
}

void CEMSocket::SetDownloadLimit(uint32 limit){
	limitenabled = true;
	if (downloadlimit == 0){
		downloadlimit = limit;
		OnReceive(0);
	}
	else
		downloadlimit = limit;
}

void CEMSocket::DisableDownloadLimit(){
	if (downloadlimit == 0 && limitenabled){
		limitenabled = false;
		OnReceive(0);
	}
	else{
		limitenabled = false;
		downloadlimit = 0;
	}
}

bool CEMSocket::SendPacket(Packet* packet, bool delpacket,bool controlpacket){
	if (!delpacket){
		Packet* copy = new Packet(packet->opcode,packet->size);
		memcpy(copy->pBuffer,packet->pBuffer,packet->size);
		packet = copy;
	}			
	if ( (!IsConnected()) || IsBusy() ){
		if (controlpacket){
			controlpacket_queue.AddTail(packet);
			return true;
		}
		else{
			standartpacket_queue.AddTail(packet);
			return true;
		}
	}
	Send(packet->DetachPacket(),packet->GetRealPacketSize());
	if (delpacket)
		delete packet;
	return true;
}

void CEMSocket::OnSend(int nErrorCode){
	if (nErrorCode){
		OnError(nErrorCode);
		return;
	}
	if (IsBusy())
		Send(0,0,0);
	if (IsBusy())
		return;
	connected = true;
	while (controlpacket_queue.GetHeadPosition() != 0 && (!IsBusy()) && IsConnected()){
		Packet* cur_packet = controlpacket_queue.GetHead();
		Send(cur_packet->DetachPacket(),cur_packet->GetRealPacketSize());
		controlpacket_queue.RemoveHead();
		delete cur_packet;
	}

	while (standartpacket_queue.GetHeadPosition() != 0 && (!IsBusy()) && IsConnected()){
		Packet* cur_packet = standartpacket_queue.GetHead();
		Send(cur_packet->DetachPacket(),cur_packet->GetRealPacketSize());
		standartpacket_queue.RemoveHead();
		delete cur_packet;
	}

}

int CEMSocket::Send(char* lpBuf,int nBufLen,int nFlags){
#define		MAXFRAGSIZE	1400
	ASSERT (sendbuffer == NULL || lpBuf == NULL );
	if (lpBuf){
		sendbuffer = lpBuf;
		sendblen = nBufLen;
		sent = 0;
	}
	while (true){
		uint32 tosend = sendblen-sent;
		if (tosend > MAXFRAGSIZE)
			tosend = MAXFRAGSIZE;
		ASSERT (tosend != 0);
		uint32 result = CAsyncSocket::Send(sendbuffer+sent,tosend);
		if (result == (uint32)SOCKET_ERROR){
			uint32 error = GetLastError();
			if (error == WSAEWOULDBLOCK){
				break;
			}
			else{
				//OnError(error);
				return -1;
			}
		}
		sent += result;
		ASSERT (sent <= sendblen);
		if (sent == sendblen){
			delete[] sendbuffer;
			sendbuffer = 0;
			sent = 0;
			sendblen = 0;
			break;
		}
	}
	return 0;
}
